unit DBTrackSource_DTEdit;

{<
  DBTrackTarget_DTEdit is design time dialog used by DBTrackSource component.

  Updated version compatible with Delphi2010+.
  Full functional demo.
  @author(Rosinsky Software - <http://www.rosinsky.cz/delphi>)
  @created(May 2006)
  @lastmod(Apr 2017)
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, db, CheckLst, ComCtrls, PropSaveMain,
  rImprovedComps;

type
  { design time dialog used by DBTrack component }
  TDBTrackSource_DTEditForm = class(TForm)
    pc: TPageControl;
    tsCol: TTabSheet;
    TabSheet1: TTabSheet;
    Label3: TLabel;
    cbKey: TrComboBoxEx;
    Label4: TLabel;
    Label5: TLabel;
    cbFormatStr: TrComboBoxEx;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    TabSheet2: TTabSheet;
    gbEmpty: TrGroupBox;
    PSFile: TPropSaveFile;
    PSMain: TPropSaveMain;
    btClose: TButton;
    gbLeft1: TrGroupBox;
    gbRight1: TrGroupBox;
    lbFields: TrListBox;
    btMove: TButton;
    btMoveAll: TButton;
    Label1: TLabel;
    Label2: TLabel;
    lbCol: TrListBox;
    btUp: TButton;
    btDown: TButton;
    btDel: TButton;
    btDelAll: TButton;
    btDelNoField: TButton;
    gbLeft2: TrGroupBox;
    Label9: TLabel;
    lbFields2: TrListBox;
    btMove2: TButton;
    btMoveAll2: TButton;
    gbRight2: TrGroupBox;
    Label10: TLabel;
    lbSpec2: TrListBox;
    btUp2: TButton;
    btDown2: TButton;
    btDel2: TButton;
    btDelAll2: TButton;
    btDelNoField2: TButton;
    procedure FormShow(Sender: TObject);
    procedure lbColDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure btUpClick(Sender: TObject);
    procedure btDownClick(Sender: TObject);
    procedure lbColKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure btDelClick(Sender: TObject);
    procedure btDelAllClick(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btMoveClick(Sender: TObject);
    procedure btMoveAllClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btDelNoFieldClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    FTracked: TStrings;
    FTrackedSpecial: TStrings;
    procedure LoadProps;
    procedure SetProps;
    procedure AddCol(FldName: string);
    procedure AddSpec(FldName: string);
    procedure MoveI(List: TListBox; old, new: Integer);
  public
  end;

{ initialize and show design time dialog }
function ShowDBTrackSourceEditor(Component: TComponent): Boolean;

implementation

{$R *.dfm}

uses TypInfo, RosiCompGlobalCfg, rtool, rdlg, rdbtool;

function ShowDBTrackSourceEditor(Component: TComponent): Boolean;
var
  F: TDBTrackSource_DTEditForm;
  DS: TDataSet;
begin
  Result:=true;
  F:=TDBTrackSource_DTEditForm.Create(Application);
  try
    F.Caption:=Component.Name;
    F.FTracked:=GetObjectProp(Component, 'TrackedFields') as TStrings;
    F.lbCol.Items.Assign(F.FTracked);
    F.FTrackedSpecial:=GetObjectProp(Component, 'TrackedSpecialFields') as TStrings;
    F.lbSpec2.Items.Assign(F.FTrackedSpecial);
    F.lbFields.Items.Clear;
    try
      DS:=GetObjectProp(Component, 'TrackedDataSet') as TDataSet;
      GetFldList(DS, F.lbFields.Items, [fdAllField]);
      F.lbFields2.Items.Assign(F.lbFields.Items);
    except
    end;
    F.cbKey.Text:=GetStrProp(Component, 'KeyFields');
    F.cbKey.Items.Assign(F.lbFields.Items);
    F.cbFormatStr.Text:=GetStrProp(Component, 'ChangesFormatStr');
    F.ShowModal;

    SetStrProp(Component, 'KeyFields', F.cbKey.Text);
    SetStrProp(Component, 'ChangesFormatStr', F.cbFormatStr.Text);
  finally
    FreeAndNil(F);
  end;
end;

procedure TDBTrackSource_DTEditForm.FormCreate(Sender: TObject);
begin
  PSFile.LoadFile(GetRosiCompIniFile);
end;

procedure TDBTrackSource_DTEditForm.FormDestroy(Sender: TObject);
begin ;
end;

procedure TDBTrackSource_DTEditForm.FormResize(Sender: TObject);
begin
  gbLeft1.Width:=pc.ClientWidth div 2;
  gbLeft2.Width:=pc.ClientWidth div 2;
end;

procedure TDBTrackSource_DTEditForm.FormShow(Sender: TObject);
begin
  pc.ActivePageIndex:=0;
  LoadProps;
end;

procedure TDBTrackSource_DTEditForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  SetProps;
end;

procedure TDBTrackSource_DTEditForm.btCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TDBTrackSource_DTEditForm.LoadProps;
begin ;
end;

procedure TDBTrackSource_DTEditForm.SetProps;
begin
  FTracked.Assign(lbCol.Items);
  FTrackedSpecial.Assign(lbSpec2.Items);
end;

// FIELDS proc
procedure TDBTrackSource_DTEditForm.AddCol(FldName: string);
var
  oldItemIndex: Integer;
begin
  oldItemIndex:=lbCol.ItemIndex;
  if (oldItemIndex < 0) or (oldItemIndex > lbCol.Count - 1) then
    oldItemIndex:=lbCol.Count - 1
  else
    Inc(oldItemIndex);
  if oldItemIndex < 0 then
    oldItemIndex:=0;
  lbCol.Items.Insert(oldItemIndex, FldName);
  lbCol.ItemIndex:=oldItemIndex;
end;

procedure TDBTrackSource_DTEditForm.AddSpec(FldName: string);
var
  oldItemIndex: Integer;
begin
  oldItemIndex:=lbSpec2.ItemIndex;
  if (oldItemIndex < 0) or (oldItemIndex > lbSpec2.Count - 1) then
    oldItemIndex:=lbSpec2.Count - 1
  else
    Inc(oldItemIndex);
  if oldItemIndex < 0 then
    oldItemIndex:=0;
  lbSpec2.Items.Insert(oldItemIndex, FldName);
  lbSpec2.ItemIndex:=oldItemIndex;
end;

procedure TDBTrackSource_DTEditForm.MoveI(List: TListBox; old, new: Integer);
begin
  if not MoveListBox(List, old, new) then
    Exit;
end;

procedure TDBTrackSource_DTEditForm.lbColDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept:=true;
  if TControl(Sender).Tag = 1 then
    MoveI(lbCol, lbCol.ItemIndex, Y div lbCol.ItemHeight + lbCol.TopIndex)
  else
    MoveI(lbSpec2, lbSpec2.ItemIndex, Y div lbSpec2.ItemHeight + lbSpec2.TopIndex);
end;

procedure TDBTrackSource_DTEditForm.btUpClick(Sender: TObject);
begin
  if TControl(Sender).Tag = 1 then
    MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex - 1)
  else
    MoveI(lbSpec2, lbSpec2.ItemIndex, lbSpec2.ItemIndex - 1)
end;

procedure TDBTrackSource_DTEditForm.btDownClick(Sender: TObject);
begin
  if TControl(Sender).Tag = 1 then
    MoveI(lbCol, lbCol.ItemIndex, lbCol.ItemIndex + 1)
  else
    MoveI(lbSpec2, lbSpec2.ItemIndex, lbSpec2.ItemIndex + 1)
end;

procedure TDBTrackSource_DTEditForm.lbColKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  L: TListBox;
begin
  if TControl(Sender).Tag = 1 then
    L:=lbCol
  else
    L:=lbSpec2;
  if (Key = VK_UP) and (ssCtrl in Shift) then
  begin
    MoveI(L, L.ItemIndex, L.ItemIndex - 1);
    Key:=0;
  end;
  if (Key = VK_DOWN) and (ssCtrl in Shift) then
  begin
    MoveI(L, L.ItemIndex, L.ItemIndex + 1);
    Key:=0;
  end;
  if (Key = VK_DELETE) then
  begin
    btDelClick(L);
    Key:=0;
  end;
end;

procedure TDBTrackSource_DTEditForm.btDelClick(Sender: TObject);
var
  i: Integer;
  L: TListBox;
begin
  if TControl(Sender).Tag = 1 then
    L:=lbCol
  else
    L:=lbSpec2;
  i:=L.ItemIndex;
  if i = -1 then
    Exit;
  L.Items.Delete(i);
  if i > L.Items.Count - 1 then
    i:=L.Items.Count - 1;
  L.ItemIndex:=i;
end;

procedure TDBTrackSource_DTEditForm.btDelAllClick(Sender: TObject);
begin
  if DlgQ('Delete all fields?') <> mrYes then
    Exit;
  if TControl(Sender).Tag = 1 then
    lbCol.Clear
  else
    lbSpec2.Clear;
end;

procedure TDBTrackSource_DTEditForm.btDelNoFieldClick(Sender: TObject);
var
  i: Integer;
  L: TListBox;
begin
  if DlgQ('Delete all orphan fields (not existing field)?') <> mrYes then
    Exit;
  if TControl(Sender).Tag = 1 then
    L:=lbCol
  else
    L:=lbSpec2;
  for i:=L.Items.Count - 1 downto 0 do
    if lbFields.Items.IndexOf(L.Items[i]) < 0 then
    begin
      L.Items.Delete(i);
    end;
end;

procedure TDBTrackSource_DTEditForm.btMoveClick(Sender: TObject);
var
  i: Integer;
  L: TListBox;
begin
  if TControl(Sender).Tag = 1 then
    L:=lbFields
  else
    L:=lbFields2;
  for i:=0 to L.Items.Count - 1 do
    if L.Selected[i] then
    begin
      if TControl(Sender).Tag = 1 then
        AddCol(L.Items[i])
      else
        AddSpec(L.Items[i]);
    end;
end;

procedure TDBTrackSource_DTEditForm.btMoveAllClick(Sender: TObject);
var
  i: Integer;
  L: TListBox;
begin
  if DlgQ('Add all columns?') <> mrYes then
    Exit;
  if TControl(Sender).Tag = 1 then
    L:=lbFields
  else
    L:=lbFields2;
  for i:=0 to L.Items.Count - 1 do
    L.Selected[i]:=true;
  btMoveClick(Sender);
end;

end.
